/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008-2009  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao.hibernate;

import net.sf.autodao.AutoDAO;
import org.springframework.beans.factory.annotation.AnnotatedBeanDefinition;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.BeanDefinitionHolder;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import static org.springframework.beans.factory.support.BeanDefinitionBuilder.genericBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.beans.factory.xml.XmlReaderContext;
import org.springframework.context.annotation.ClassPathBeanDefinitionScanner;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import static org.springframework.util.StringUtils.commaDelimitedListToStringArray;
import org.w3c.dom.Element;

/**
 * Processes <code>dao:hibernateScan</code> elements and builds Spring beans.
 * <p/>
 * Creates {@link HibernateDao} beans and wrap it with {@link HibernateDaoProxyFactoryBean} so that AutoDAO magic
 * works.
 * <p/>
 * NOT FOR PUBLIC USE.
 */
public final class HibernateScanBeanDefinitionParser implements BeanDefinitionParser {
  private static final String BASE_PACKAGE_ATTRIBUTE = "base-package";
  private static final String RESOURCE_PATTERN_ATTRIBUTE = "resource-pattern";

  @Override
  public BeanDefinition parse(final Element element, final ParserContext parserContext) {
    final HibernateDaoScanner scanner = new HibernateDaoScanner(parserContext.getRegistry());
    final XmlReaderContext xmlReaderContext = parserContext.getReaderContext();
    if (element.hasAttribute(RESOURCE_PATTERN_ATTRIBUTE)) {
      scanner.setResourcePattern(element.getAttribute(RESOURCE_PATTERN_ATTRIBUTE));
    }
    scanner.setResourceLoader(xmlReaderContext.getResourceLoader());
    scanner.setBeanDefinitionDefaults(parserContext.getDelegate().getBeanDefinitionDefaults());
    scanner.setAutowireCandidatePatterns(parserContext.getDelegate().getAutowireCandidatePatterns());
    scanner.scan(
        xmlReaderContext,
        ParserUtils.parseSessionFactory(element, xmlReaderContext),
        commaDelimitedListToStringArray(element.getAttribute(BASE_PACKAGE_ATTRIBUTE)));
    return null;
  }

  private static class HibernateDaoScanner extends ClassPathBeanDefinitionScanner {

    public HibernateDaoScanner(final BeanDefinitionRegistry registry) {
      super(registry, true);
    }

    protected final void scan(final XmlReaderContext ctx,
                              final String sessionFactoryBeanName,
                              final String[] basePackages) {
      for (final String basePackage : basePackages) {
        for (final BeanDefinition candidate : findCandidateComponents(basePackage)) {
          final String daoInterface = candidate.getBeanClassName();
          final AbstractBeanDefinition definition = genericBeanDefinition("net.sf.autodao.hibernate.HibernateDaoProxyFactoryBean")
              .addConstructorArgValue(daoInterface)
              .addConstructorArgReference(sessionFactoryBeanName)
              .addConstructorArgValue(null)
              .addConstructorArgValue(ParserUtils.getOrRegisterFinderAdvisor(ctx))
              .getBeanDefinition();
          final String beanName = "net.sf.autodao.AutoDAO[" + daoInterface + ']';
          postProcessBeanDefinition(definition, beanName);
          if (checkCandidate(beanName, definition)) {
            registerBeanDefinition(new BeanDefinitionHolder(definition, beanName), getRegistry());
          }
        }
      }
    }

    @Override
    protected void registerDefaultFilters() {
      addIncludeFilter(new AnnotationTypeFilter(AutoDAO.class, false));
    }

    @Override
    protected boolean isCandidateComponent(final AnnotatedBeanDefinition beanDefinition) {
      return beanDefinition.getMetadata().isInterface();
    }
  }
}
